unit VDRE_Config;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ComCtrls, RefCtrls, NumEdits;

type
  TConfigDialog = class(TForm)
    OKButton: TButton;
    CancelButton: TButton;
    ApplyButton: TButton;
    OptionPage: TPageControl;
    ViewerTab: TTabSheet;
    ImageGroup: TGroupBox;
    PluginPathLbl: TLabel;
    PluginPathEdt: TRefDirectoryEdit;
    TextGroup: TGroupBox;
    TabWidthLbl: TLabel;
    TabWidthEdt: TUpDownEdit;
    FontLbl: TLabel;
    FontBtn: TButton;
    FormatTab: TTabSheet;
    AssociationLbl: TLabel;
    AssociationLst: TListView;
    AllImageChk: TCheckBox;
    AllTextChk: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure ApplyButtonClick(Sender: TObject);
    procedure PluginPathEdtChange(Sender: TObject);
    procedure TabWidthEdtVerify(Sender: TObject; var AText: string; var APosition: Integer;
      Relation: TNumericallyRelationship);
    procedure FontBtnClick(Sender: TObject);
    procedure AssociationLstChange(Sender: TObject; Item: TListItem; Change: TItemChange);
    procedure AllImageChkClick(Sender: TObject);
    procedure AllTextChkClick(Sender: TObject);
  private
    { Private 錾 }
    FUpdating: Integer;
    procedure ReadOption;
    procedure WriteOption;
    function IsTextFile(FileType, PerceivedType: string): Boolean;
    function IsPluginSupported(Extension: string): Boolean;
    function GetPluginExplanation(Extension: string): string;
    procedure AddItem(Explanation, Extension, PerceivedType: string; Associated: Boolean);
    procedure ReadExtension;
    procedure WriteExtension;
    procedure UpdateChkState;
  public
    { Public 錾 }
  end;

var
  ConfigDialog: TConfigDialog;

implementation

{$R *.dfm}

uses
  VDRE_Main,
  System.Types, System.Win.Registry, System.StrUtils, Winapi.ShlObj, Winapi.CommCtrl;

resourcestring
  SDisplayVidere = 'Videre ŕ\(&D)';

procedure TConfigDialog.ReadOption;
begin
  PluginPathEdt.Text := MainForm.PluginPath;
  TabWidthEdt.Value := MainForm.TabWidth;
  FontBtn.Font := MainForm.ViewerFont;
  FontBtn.Caption := MainForm.ViewerFont.Name;
end;

procedure TConfigDialog.WriteOption;
begin
  MainForm.PluginPath := PluginPathEdt.Text;
  MainForm.TabWidth := TabWidthEdt.Value;
  MainForm.ViewerFont := FontBtn.Font;
end;

function TConfigDialog.IsTextFile(FileType, PerceivedType: string): Boolean;
begin
  Result := SameText('txtfile', FileType)
    or (('' <> FileType) and SameText('text', PerceivedType));
end;

function TConfigDialog.IsPluginSupported(Extension: string): Boolean;
begin
  Result := 0 <= MainForm.Plugins.IndexOfName(Extension);
end;

function TConfigDialog.GetPluginExplanation(Extension: string): string;
var
  PluginHnd: THandle;
  GetPluginInfo: TGetPluginInfo;
  Buffer: array[0..MAX_PATH] of AnsiChar;
begin
  Result := '';

  PluginHnd := LoadLibrary(PChar(MainForm.Plugins.Values[Extension]));
  if PluginHnd <> 0 then
  begin
    @GetPluginInfo := GetProcAddress(PluginHnd, 'GetPluginInfo');
    if @GetPluginInfo <> nil then
    begin
      GetPluginInfo(0, Buffer, MAX_PATH);
      if Buffer = '00IN' then
      begin
        GetPluginInfo(3, Buffer, MAX_PATH);
        Result := string(Buffer);
      end;
    end;
    FreeLibrary(PluginHnd);
  end;
end;

procedure TConfigDialog.AddItem(Explanation, Extension, PerceivedType: string; Associated: Boolean);
var
  Item: TListItem;
begin
  Item := AssociationLst.FindCaption(0, Explanation, False, True, False);
  if Item = nil then
  begin
    Item := AssociationLst.Items.Add;
    with Item do
    begin
      Caption := Explanation;
      SubItems.Add(Extension);
      SubItems.Add(PerceivedType);
      Checked := Associated;
    end;
  end
  else
    Item.SubItems.Strings[0] := Item.SubItems.Strings[0] + ';' + Extension;
end;

procedure TConfigDialog.ReadExtension;
var
  Registry: TRegistry;
  Associations: TStrings;
  i: Integer;
  Extension,
  FileType,
  PerceivedType,
  Explanation: string;
  Associated: Boolean;
begin
  AssociationLst.Items.Clear;
  Registry := TRegistry.Create;
  Associations := TStringList.Create;
  try
    Registry.RootKey := HKEY_CLASSES_ROOT;
    Registry.OpenKeyReadOnly('');
    Registry.GetKeyNames(Associations);
    Registry.CloseKey;
    for i := 0 to MainForm.Plugins.Count - 1 do
    begin
      Extension := MainForm.Plugins.Names[i];
      if 0 > Associations.IndexOf(Extension) then
        Associations.Add(Extension);
    end;
    for i := 0 to Associations.Count - 1 do
    begin
      Extension := Associations.Strings[i];
      if Extension[1] = '.' then
      begin
        FileType := '';
        PerceivedType := '';
        if Registry.OpenKeyReadOnly(Extension) then
        begin
          FileType := Registry.ReadString('');
          PerceivedType := Registry.ReadString('PerceivedType');
          Registry.CloseKey;
        end;
        if IsTextFile(FileType, PerceivedType) or IsPluginSupported(Extension) then
        begin
          if PerceivedType = '' then
          begin
            if IsPluginSupported(Extension) then
              PerceivedType := 'image'
            else if IsTextFile(FileType, PerceivedType) then
              PerceivedType := 'text';
          end;
          Explanation := '';
          Associated := False;
          if Registry.OpenKeyReadOnly(FileType) then
          begin
            Explanation := Registry.ReadString('');
            Associated := Registry.KeyExists('shell\videre\command');
            Registry.CloseKey;
          end;
          if '' = Explanation then
          begin
            Explanation := GetPluginExplanation(Extension);
            if '' = Explanation then
              Explanation := FileType;
          end;
          AddItem(Explanation, Extension, PerceivedType, Associated);
        end;
      end;
    end;
  finally
    Registry.Free;
    Associations.Free;
  end;
end;

procedure TConfigDialog.WriteExtension;
var
  Registry: TRegistry;
  ExtArray: TStringDynArray;
  FileType: string;
  i, j: Integer;
  Change,
  KeyEmpty: Boolean;
begin
  Change := False;
  Registry := TRegistry.Create;
  try
    for i := 0 to AssociationLst.Items.Count - 1 do
    begin
      ExtArray := SplitString(AssociationLst.Items.Item[i].SubItems.Strings[0], ';');
      for j := 0 to Length(ExtArray) - 1 do
      begin
        //Vista ȍ~ł͊ǗҌȂꍇAHKEY_CLASSES_ROOT  KEY_READ ̂ݗLB
        Registry.RootKey := HKEY_CLASSES_ROOT;
        FileType := '';
        if Registry.OpenKeyReadOnly(ExtArray[j]) then
        begin
          FileType := Registry.ReadString('');
          Registry.CloseKey;
        end;
        if FileType = '' then
          FileType := 'Videre' + ExtArray[0] + 'File';
        Registry.Access := KEY_ALL_ACCESS;  //OpenKeyReadOnly ł̕ύXɖ߂B
        Registry.RootKey := HKEY_CURRENT_USER;
        if AssociationLst.Items.Item[i].Checked then
        begin
          if Registry.OpenKey('Software\Classes\' + ExtArray[j], True) then
          begin
            Registry.WriteString('', FileType);
            Registry.CloseKey;
          end;
          if Registry.OpenKey('Software\Classes\' + FileType, True) then
          begin
            Registry.WriteString('', AssociationLst.Items.Item[i].Caption);
            if Registry.OpenKey('shell\videre', True) then
            begin
              Registry.WriteString('', SDisplayVidere);
              if Registry.OpenKey('command', True) then
                Registry.WriteString('', '"' + ParamStr(0) + '" "%1"');
            end;
            Registry.CloseKey;
          end;
          Change := True;
        end
        else
        begin
          KeyEmpty := not Registry.KeyExists('Software\Classes\' + FileType);
          if Registry.OpenKey('Software\Classes\' + FileType + '\shell', False) then
          begin
            if Registry.DeleteKey('videre') then
              Change := True;
            KeyEmpty := not Registry.HasSubKeys;
            Registry.CloseKey;
            if KeyEmpty then
            begin
              KeyEmpty := False;
              if Registry.OpenKey('Software\Classes\' + FileType, False) then
              begin
                Registry.DeleteKey('shell');
                KeyEmpty := not Registry.HasSubKeys;
                Registry.CloseKey;
              end;
              if KeyEmpty then
                Registry.DeleteKey('Software\Classes\' + FileType);
            end;
          end;
          if KeyEmpty then
            Registry.DeleteKey('Software\Classes\' + ExtArray[j]);
        end;
      end;
    end;
  finally
    Registry.Free;
    if Change then
      SHChangeNotify(SHCNE_ASSOCCHANGED, SHCNF_FLUSH, nil, nil);
  end;
end;

procedure TConfigDialog.UpdateChkState;
var
  Item: TListItem;
  ImageCount, ImageCheck, TextCount, TextCheck: Integer;
begin
  ImageCount := 0;
  ImageCheck := 0;
  TextCount := 0;
  TextCheck := 0;
  for Item in AssociationLst.Items do
  begin
    if Item.SubItems.Strings[1] = 'image' then
    begin
      Inc(ImageCount);
      if Item.Checked then
        Inc(ImageCheck);
    end;

    if Item.SubItems.Strings[1] = 'text' then
    begin
      Inc(TextCount);
      if Item.Checked then
        Inc(TextCheck);
    end;
  end;

  if ImageCheck = 0 then
    AllImageChk.State := cbUnchecked
  else if ImageCount = ImageCheck then
    AllImageChk.State := cbChecked
  else
    AllImageChk.State := cbGrayed;

  if TextCheck = 0 then
    AllTextChk.State := cbUnchecked
  else if TextCount = TextCheck then
    AllTextChk.State := cbChecked
  else
    AllTextChk.State := cbGrayed;
end;

procedure TConfigDialog.FormCreate(Sender: TObject);
begin
  FUpdating := 0;

  {$IFDEF RELEASE}
  OptionPage.TabIndex := 0;
  {$ENDIF RELEASE}
end;

procedure TConfigDialog.FormShow(Sender: TObject);
begin
  Inc(FUpdating);
  try
    ReadOption;
    ReadExtension;
    UpdateChkState;
  finally
    Dec(FUpdating);
  end;
  ApplyButton.Enabled := False;
end;

procedure TConfigDialog.OKButtonClick(Sender: TObject);
begin
  if ApplyButton.Enabled then
    ApplyButton.Click;
  Close;
end;

procedure TConfigDialog.CancelButtonClick(Sender: TObject);
begin
  Close;
end;

procedure TConfigDialog.ApplyButtonClick(Sender: TObject);
begin
  WriteOption;
  WriteExtension;
  ApplyButton.Enabled := False;
end;

procedure TConfigDialog.PluginPathEdtChange(Sender: TObject);
begin
  if FUpdating > 0 then Exit;

  Inc(FUpdating);
  try
    MainForm.InitPlugins(PluginPathEdt.Text);
    ReadExtension;
    ApplyButton.Enabled := True;
  finally
    Dec(FUpdating);
  end;
end;

procedure TConfigDialog.TabWidthEdtVerify(Sender: TObject; var AText: string;
  var APosition: Integer; Relation: TNumericallyRelationship);
begin
  if Relation <> nrAccord then
  begin
    Beep;
    AText := IntToStr(APosition);
  end
  else
    ApplyButton.Enabled := True;
end;

procedure TConfigDialog.FontBtnClick(Sender: TObject);
var
  Dialog: TFontDialog;
begin
  Dialog := TFontDialog.Create(Self);
  try
    Dialog.Font := FontBtn.Font;
    if Dialog.Execute then
    begin
      FontBtn.Font := Dialog.Font;
      FontBtn.Caption := Dialog.Font.Name;
      ApplyButton.Enabled := True;
    end;
  finally
    Dialog.Free;
  end;
end;

procedure TConfigDialog.AssociationLstChange(Sender: TObject; Item: TListItem; Change: TItemChange);
var
  Checked: Boolean;
begin
  if FUpdating > 0 then Exit;

  Inc(FUpdating);
  try
    //TListItem.Checked ̕ύX𒲂ׂB
    if Change = ctState then
    begin
      Checked := ListView_GetItemState(AssociationLst.Handle,
        Item.Index, LVIS_STATEIMAGEMASK) shr 12 = 2;
      if (Item.StateIndex = 1) <> Checked then
      begin
        if Checked then
          Item.StateIndex := 1
        else
          Item.StateIndex := 0;
        ApplyButton.Enabled := True;
      end;
    end;
    UpdateChkState;
  finally
    Dec(FUpdating);
  end;
end;

procedure TConfigDialog.AllImageChkClick(Sender: TObject);
var
  Item: TListItem;
begin
  if FUpdating > 0 then Exit;

  Inc(FUpdating);
  try
    for Item in AssociationLst.Items do
    begin
      if Item.SubItems.Strings[1] = 'image' then
        Item.Checked := AllImageChk.Checked;
    end;
    UpdateChkState;
  finally
    Dec(FUpdating);
  end;
  ApplyButton.Enabled := True;
end;


procedure TConfigDialog.AllTextChkClick(Sender: TObject);
var
  Item: TListItem;
begin
  if FUpdating > 0 then Exit;

  Inc(FUpdating);
  try
    for Item in AssociationLst.Items do
    begin
      if Item.SubItems.Strings[1] = 'text' then
        Item.Checked := AllTextChk.Checked;
    end;
    UpdateChkState;
  finally
    Dec(FUpdating);
  end;
  ApplyButton.Enabled := True;
end;

end.
